#
# Virtual Environments, Reykjavik University
# Updated room code with autoshading, 2009
# hannes@ru.is
#
# Basic scene with lighting and textured surfaces
#
# Big thanks to Martins Upitis for the amazing Bathroom Blender demo, from which 
# I borrowed the models and textures. 
# Also thanks to Palli and Palmi for the Movement class.
#
import direct.directbase.DirectStart
from pandac.PandaModules import *
from random import choice
from direct.showbase.DirectObject import DirectObject
from direct.showbase import Audio3DManager
from direct.gui.OnscreenText import OnscreenText
from direct.filter.CommonFilters import CommonFilters

from direct.interval.MetaInterval import Sequence
from direct.interval.LerpInterval import LerpHprInterval

from movement import Movement

wall_data = [(0,0,0),(1,0,0),(2,0,-90),(2,-1,0),(3,-1,-90),(3,-2,-90),(3,-3,-180),(2,-3,-180),(1,-3,-180),(0,-3,-270),(0,-2,-270),(0,-1,-270)]
floor_data = [(0,-1),(1,-1),(0,-2),(1,-2),(2,-2),(0,-3),(1,-3),(2,-3)]
ceiling_data = [(0,0,False),(1,0,True),(0,-1,True),(1,-1,False),(2,-1,False),(0,-2,False),(1,-2,False),(2,-2,True)]

class Room:
    
    def __init__(self, parent_pathnode, wall_data, floor_data, ceiling_data):
        """
        Generate a room with walls, floor and a ceiling from lists of 1x1 panel
        locations and orientations.
            'parent_pathnode' = The pathnode immediately above this one in the scene graph
            'wall_data' = A list of wall panel touples: (x location, y location, angle)
            'floor_data' = A list of floor panel touples: (x location, y location)
            'ceiling_data' = A list of ceiling panel touples: (x location , y location, light?)
        """
        self.parent = parent_pathnode
        self.wall_data = wall_data
        self.floor_data = floor_data
        self.ceiling_data = ceiling_data
        self.grid_panel = loader.loadModel("models/grid-sparse.egg")
        self.grid_panel.setTransparency(TransparencyAttrib.MAlpha)
        self.create_floor()
        self.create_ceiling()
        self.create_walls()
        self.create_lights()
        
    def create_walls(self):
        """ Create the walls according to Room data, with randomized panel textures """
        
        # WALL MATERIAL
        wallmaterial = Material()
        wallmaterial.setShininess(10.0) #Make this material shiny
        wallmaterial.setSpecular(VBase4(1,1,1,1))
        wallmaterial.setDiffuse(VBase4(0.7,0.7,0.7,1))            
        # WALL TEXTURE
        walltexture  = loader.loadTexture("models/textures/tiles_wall_tex.png")
        wallnormmap  = loader.loadTexture("models/textures/tiles_wall_nm.tga")
        
        for wallpanel_data in self.wall_data:
            wallpanel = self.parent.attachNewNode("wall")
            self.grid_panel.instanceTo(wallpanel) 
            wallpanel.setPos(wallpanel_data[0],wallpanel_data[1],0)
            wallpanel.setHpr(wallpanel_data[2],0,0)
            wallpanel.setMaterial(wallmaterial,1)

            texture_stage = TextureStage('wall texture')
            texture_stage.setMode(TextureStage.MModulate)
            wallpanel.setTexture(texture_stage, walltexture)

            normmap_stage = TextureStage('wall normal map')
            normmap_stage.setMode(TextureStage.MNormal)
            wallpanel.setTexture(normmap_stage, wallnormmap)            

                   
    def create_floor(self):
        """ Create the floor according to Room data """
 
        # FLOOR MATERIAL
        floormaterial = Material()
        floormaterial.setShininess(5.0) #Make this material shiny
        floormaterial.setSpecular(VBase4(1,1,1,1))
        floormaterial.setDiffuse(VBase4(0.5,0.5,0.6,1))        
        # FLOOR TEXTURE
        floortexture = loader.loadTexture("models/textures/floor_tex.tga")
        floornormmap = loader.loadTexture("models/textures/floor_nm.tga")
        floorspecmap = loader.loadTexture("models/textures/floor_spec.tga")

        for floorpanel_data in self.floor_data:
            floorpanel = self.parent.attachNewNode("floor")
            self.grid_panel.instanceTo(floorpanel)
            floorpanel.setPos(floorpanel_data[0],floorpanel_data[1],0)
            floorpanel.setHpr(0,-90,0)
            floorpanel.setMaterial(floormaterial,1)

            texture_stage = TextureStage('floor texture')
            texture_stage.setMode(TextureStage.MModulate)
            floorpanel.setTexture(texture_stage, floortexture)
            
            normmap_stage = TextureStage('floor normal map')
            normmap_stage.setMode(TextureStage.MNormal)
            floorpanel.setTexture(normmap_stage, floornormmap)
      
            specmap_stage = TextureStage('floor specular map')
            specmap_stage.setMode(TextureStage.MGloss)
            floorpanel.setTexture(specmap_stage, floorspecmap)  
                    
    def create_ceiling(self):
        """ Create the ceiling, with ceiling light textures, according to Room data """      
        
        # CEILING MATERIAL
        ceilingmaterial = Material()
        ceilingmaterial.setShininess(0.0) #Make this material matt
        ceilingmaterial.setSpecular(VBase4(0,0,0,1))
        ceilingmaterial.setDiffuse(VBase4(0.3,0.3,0.35,1))        
        # CEILING TEXTURE
        ceiling_clear = loader.loadTexture("models/textures/ceiling_tanned.png")
        
        for ceilingpanel_data in self.ceiling_data:
            ceilingpanel = self.parent.attachNewNode("ceiling")
            self.grid_panel.instanceTo(ceilingpanel)
            ceilingpanel.setPos(ceilingpanel_data[0],ceilingpanel_data[1],1)
            ceilingpanel.setHpr(0,90,0)
            ceilingpanel.setMaterial(ceilingmaterial,1)
            ceilingpanel.setTexture(ceiling_clear) 
        
    def create_lights(self):
        """ Add light fixtures and point light sources """
        
        # BULB MATERIAL
        bulbmaterial = Material()
        bulbmaterial.setEmission(VBase4(1,1,1,1))
        # BULB MODEL
        self.lightbulb = loader.loadModel("models/bulbfixture.egg")
        self.bulb = self.lightbulb.find("**/Sphere.001")
        self.bulb.setMaterial(bulbmaterial,1)

        # GLOW MAP added to the light bulb
        glowmap = loader.loadTexture("models/textures/light.png")
        glowmap_stage = TextureStage('glow map')
        glowmap_stage.setMode(TextureStage.MModulateGlow)
        self.bulb.setTexture(glowmap_stage, glowmap,1)
        # end GLOW MAP

        self.lightfixtures = []
      
        
        for ceilingpanel_data in self.ceiling_data:

            if ceilingpanel_data[2] is True:
                # LIGHT FIXTURE
                lightfixture = self.parent.attachNewNode("bulb")
                self.lightbulb.instanceTo(lightfixture)
                lightfixture.setScale(0.1)
                lightfixture.setPos(Vec3(ceilingpanel_data[0]+0.5,ceilingpanel_data[1]-0.5,1))
                self.lightfixtures.append(lightfixture)
                swing = Sequence(LerpHprInterval(lightfixture,3,Vec3(0,20,0), Vec3(0,-20,0), blendType="easeInOut"),LerpHprInterval(lightfixture,3,Vec3(0,-20,0), Vec3(0,20,0), blendType="easeInOut"))
                swing.loop()
                # POINT LIGHT
                point_source = PointLight('pointlight')
                point_source.setColor(Vec4(0.6,0.6,0.6,1))
                point_source.setAttenuation(Vec3(0.3,0.3,1.1))
                point = lightfixture.attachNewNode(point_source)
                point.setPos(0,0,-5)
                render.setLight(point)

class World(DirectObject):

    def __init__(self):
        

        # START AUTO SHADING
        render.setShaderAuto()
        self.shaderstate = True
        render.setAntialias(AntialiasAttrib.MAuto)
        # INCLUDE AUTO SCREEN EFFECTS
        self.filters = CommonFilters(base.win, base.cam)
        ##self.filters.setAmbientOcclusion()    # Uncomment for a great but slow "shadow in corners effect"    
        self.filters.setBloom(blend=(0.3,0.4,0.3,0.5), size="large")
        
        self.accept('p', self.set_shaderstate)
        self.accept('f', self.set_wireframe)
        
        # Build a room
        self.room_pathnode = render.attachNewNode("Room")
        self.room = Room(self.room_pathnode, wall_data, floor_data, ceiling_data)

        # Props inside the room
        self.bathtub = loader.loadModel("models/bathtub.egg")
        self.bathtub.reparentTo(render)
        self.bathtub.setScale(0.1)
        self.bathtub.setPos(1.5,-0.2,0)
        bathtubmaterial = Material()
        bathtubmaterial.setShininess(20.0) #Make this material shiny
        bathtubmaterial.setSpecular(VBase4(1,1,1,1))
        bathtubmaterial.setDiffuse(VBase4(1,1,1,1))
        self.bathtub.setMaterial(bathtubmaterial ,1)
        self.sink = loader.loadModel("models/sink.egg")
        self.sink.reparentTo(render)
        self.sink.setScale(0.11)
        self.sink.setPos(1.93,-0.45,0.25)
        self.sink.setH(180)
        self.pipes = loader.loadModel("models/pipes.egg")
        self.pipes.reparentTo(render)
        self.pipes.setScale(0.11)
        self.pipes.setPos(1.97,-0.4,0.31)
        self.pipes.setH(180)
        
        # Camera setup
        base.disableMouse()
        base.camera.setPos(-0.5, -3.5, 0.5)
        base.camera.setHpr(-45,0,0)  
        base.camLens.setFov(50)  
        base.camLens.setNear(0.2)
        base.camLens.setFar(5)

        self.text = OnscreenText(pos = (0, -0.9), fg = (1,1,1,1), bg = (0.3, 0.3, 0.3, 0.5), align = TextNode.ACenter, scale = 0.07, mayChange = True)   

    def set_shaderstate(self):

        if not self.shaderstate:
            render.setShaderAuto()
            self.filters.setBloom(blend=(0.3,0.4,0.3,0.5), size="large")  
            ##self.filters.setAmbientOcclusion()
            self.text.setText("Hardware shaders are ON")
        else:
            render.setShaderOff()
            self.filters.delBloom()
            ##self.filters.delAmbientOcclusion()
            self.text.setText("Hardware shaders are OFF")
        self.shaderstate = not self.shaderstate
        
    def set_wireframe(self):
        base.toggleWireframe()


 
# Create a world
w = World()
m = Movement(showCoordinates=False)

# Start main loop
run()
